@@ -81,6 +81,8 @@ sub connect_database {
 	my $info;
 	my $olddir = getcwd;
 	my $debug = $ENV{DBDPG_DEBUG} || 0;
+	delete @ENV{ 'PGSERVICE', 'PGDATABASE' };
+
 
 	## We'll try various ways to get to a database to test with
 
@@ -235,10 +237,6 @@ version: $version
 		$testuser = 'postgres';
 	}
 
-    # non-ASCII parts of the tests assume UTF8
-    $testdsn =~ s/;?\bclient_encoding=[^;]+//;
-    $testdsn .= ';client_encoding=utf8';
-
 	## From here on out, we don't return directly, but save it first
   GETHANDLE: {
 		eval {
@@ -287,7 +285,9 @@ version: $version
 		$helpconnect = 16;
 
 		## Use the initdb found by App::Info
-		$initdb = $ENV{DBDPG_INITDB} || $ENV{PGINITDB} || '';
+        if (! length $initdb or $initdb eq 'default') {
+            $initdb = $ENV{DBDPG_INITDB} || $ENV{PGINITDB} || '';
+        }
 		if (!$initdb or ! -e $initdb) {
 			$initdb = 'initdb';
 		}
@@ -320,6 +320,9 @@ version: $version
 		elsif ($info =~ /(\d+\.\d+)/) {
 			$version = $1;
 		}
+		elsif ($info =~ /(\d+)(?:devel|beta|rc|alpha)/) { ## Can be 10devel
+			$version = $1;
+		}
 		else {
 			die "No version from initdb?! ($info)\n";
 		}
@@ -339,10 +342,12 @@ version: $version
 		}
 
 		## initdb and pg_ctl seems to be available, let's use them to fire up a cluster
-		warn "Please wait, creating new database for testing\n";
+        warn "Please wait, creating new database (version $version) for testing\n";
 		$info = '';
 		eval {
-			$info = qx{$initdb --locale=C -E UTF8 -D $testdir/data 2>&1};
+            my $com = "$initdb --locale=C -E UTF8 -D $testdir/data";
+			$debug and warn" Attempting: $com\n";
+			$info = qx{$com 2>&1};
 		};
 		last GETHANDLE if $@; ## Fail - initdb bad
 
@@ -420,6 +425,9 @@ version: $version
 
 		## Attempt to boost the system oids above an int for certain testing
 		(my $resetxlog = $initdb) =~ s/initdb/pg_resetxlog/;
+		if ($version >= 10) {
+			$resetxlog =~ s/pg_resetxlog/pg_resetwal/;
+		}
 		eval {
 			$info = qx{$resetxlog --help};
 		};
@@ -493,7 +501,7 @@ version: $version
 			$debug and diag qq{Writing to "$conf"};
 			print $cfh "\n\n## DBD::Pg testing parameters\n";
 			print $cfh "port=$testport\n";
-			print $cfh "max_connections=5\n";
+			print $cfh "max_connections=11\n";
 			if ($version >= 8.0) {
 				print $cfh "log_statement = 'all'\n";
 				print $cfh "log_line_prefix = '%m [%p] '\n";
@@ -512,6 +520,21 @@ version: $version
 			print $cfh "log_filename = 'postgres%Y-%m-%d.log'\n";
 			print $cfh "log_rotation_size = 0\n";
 
+			if ($version >= 9.4) {
+				print $cfh "wal_level = logical\n";
+				print $cfh "max_replication_slots = 1\n";
+				print $cfh "max_wal_senders = 1\n";
+
+				open my $hba, '>>', "$testdir/data/pg_hba.conf"
+					or die qq{Could not open "$testdir/data/pg_hba.conf": $!\n};
+
+				print $hba "local\treplication\tall\ttrust\n";
+				print $hba "host\treplication\tall\t127.0.0.1/32\ttrust\n";
+				print $hba "host\treplication\tall\t::1/128\ttrust\n";
+
+				close $hba or die qq{Could not close "$testdir/data/pg_hba.conf": $!\n};
+			}
+
 			print $cfh "listen_addresses='127.0.0.1'\n" if $^O =~ /Win32/;
 			print $cfh "\n";
 			close $cfh or die qq{Could not close "$conf": $!\n};
@@ -554,7 +577,7 @@ version: $version
 		}
 
 		## Attempt to connect to this server
-		$testdsn = "dbi:Pg:dbname=postgres;client_encoding=utf8;port=$testport";
+		$testdsn = "dbi:Pg:dbname=postgres;port=$testport";
 		if ($^O =~ /Win32/) {
 			$testdsn .= ';host=localhost';
 		}
@@ -651,6 +674,12 @@ version: $version
 
 	$debug and diag "Got a database handle ($dbh)";
 
+	if (!$arg->{quickreturn} or 1 != $arg->{quickreturn}) {
+		## non-ASCII parts of the tests assume UTF8
+		$dbh->do('SET client_encoding = utf8');
+		$dbh->{pg_enable_utf8} = -1;
+	}
+
 	if ($arg->{quickreturn}) {
 		$debug and diag 'Returning via quickreturn';
 		return $helpconnect, '', $dbh;
@@ -754,13 +783,20 @@ sub get_test_settings {
 
 	## Find the best candidate for the pg_ctl program
 	my $pg_ctl = 'pg_ctl';
-	if (exists $ENV{DBDPG_INITDB} and -e $ENV{DBDPG_INITDB}) {
+    my $initdb = 'default';
+    if (exists $ENV{POSTGRES_HOME} and -e "$ENV{POSTGRES_HOME}/bin/pg_ctl") {
+        $pg_ctl = "$ENV{POSTGRES_HOME}/bin/pg_ctl";
+        $initdb = "$ENV{POSTGRES_HOME}/bin/initdb";
+    }
+	elsif (exists $ENV{DBDPG_INITDB} and -e $ENV{DBDPG_INITDB}) {
 		($pg_ctl = $ENV{DBDPG_INITDB}) =~ s/initdb/pg_ctl/;
-	} elsif (exists $ENV{PGINITDB} and -e $ENV{PGINITDB}) {
+	}
+    elsif (exists $ENV{PGINITDB} and -e $ENV{PGINITDB}) {
 		($pg_ctl = $ENV{PGINITDB}) =~ s/initdb/pg_ctl/;
 	}
+
 	my ($testdsn, $testuser, $testdir, $error) = ('','','','?');
-	my ($helpconnect, $su, $uid, $initdb, $version) = (0,'','','default',0);
+	my ($helpconnect, $su, $uid, $version) = (0,'','',0);
 	my $inerror = 0;
 	if (-e $helpfile) {
 		open $fh, '<', $helpfile or die qq{Could not open "$helpfile": $!\n};
