@@ -26,7 +26,7 @@ methods defined by App::Info::RDBMS. Methods that trigger events will trigger
 them only the first time they're called (See L<App::Info|App::Info> for
 documentation on handling events). To start over (after, say, someone has
 installed PostgreSQL) construct a new App::Info::RDBMS::PostgreSQL object to
-aggregate new metadata.
+aggregate new meta data.
 
 Some of the methods trigger the same events. This is due to cross-calling of
 shared subroutines. However, any one event should be triggered no more than
@@ -42,7 +42,7 @@ use App::Info::RDBMS;
 use App::Info::Util;
 use vars qw(@ISA $VERSION);
 @ISA = qw(App::Info::RDBMS);
-$VERSION = '0.45';
+$VERSION = '0.57';
 use constant WIN32 => $^O eq 'MSWin32';
 
 my $u = App::Info::Util->new;
@@ -135,16 +135,16 @@ sub new {
     my @paths = $self->search_bin_dirs;
     my @exes = $self->search_exe_names;
 
-    if (my $cfg = $u->first_cat_exe(\@exes, @paths) and !$ENV{DBDPG_TESTINITDB}) {
+    if (my $cfg = $u->first_cat_exe(\@exes, @paths)) {
         # We found it. Confirm.
-        $self->{pg_config} = $self->confirm( key      => 'pg_config',
+        $self->{pg_config} = $self->confirm( key      => 'path to pg_config',
                                              prompt   => "Path to pg_config?",
                                              value    => $cfg,
                                              callback => sub { -x },
                                              error    => 'Not an executable');
     } else {
         # Handle an unknown value.
-        $self->{pg_config} = $self->unknown( key      => 'pg_config',
+        $self->{pg_config} = $self->unknown( key      => 'path to pg_config',
                                              prompt   => "Path to pg_config?",
                                              callback => sub { -x },
                                              error    => 'Not an executable');
@@ -268,15 +268,25 @@ my $get_version = sub {
             # Beta/devel/release candidates are treated as patch level "0"
             @{$self}{qw(version major minor patch)} =
               ($version, $x, $y, $z);
-        } elsif ($version =~ /(\d+)\.(\d+)/) {
+        }
+        elsif ($version =~ /^(\d)\.(\d+)/) {        # < v10
             # New versions, such as "7.4", are treated as patch level "0"
             @{$self}{qw(version major minor patch)} =
-              ($version, $1, $2, 0);
-        } else {
-            $self->error("Failed to parse PostgreSQL version parts from " .
-                         "string '$version'");
+                ($version, $1, $2, 0);
         }
-    } else {
+        elsif ($version =~ /^(\d{2,})\.(\d+)/) {    # >= v10
+            @{$self}{qw(version major minor patch)} =
+                ($version, $1, 0, $2); # from v10 onwards, $2 will be patch level
+        }
+        elsif ($version =~ /^(\d{2,})(devel|beta|rc|alpha)/) {
+            @{$self}{qw(version major minor patch)} =
+                ($version, $1, 0, 0);
+        }
+        else {
+            $self->error("Failed to parse PostgreSQL version parts from string '$version'");
+        }
+    }
+    else {
         $self->error("Unable to parse version from string '$data'");
     }
 };
@@ -289,7 +299,7 @@ sub name {
     $get_version->($self) unless $self->{'--version'};
 
     # Handle an unknown name.
-    $self->{name} ||= $self->unknown( key => 'name' );
+    $self->{name} ||= $self->unknown( key => 'postgres name' );
 
     # Return the name.
     return $self->{name};
@@ -342,7 +352,13 @@ sub version {
         # Create a validation code reference.
         my $chk_version = sub {
             # Try to get the version number parts.
-            my ($x, $y, $z) = /^(\d+)\.(\d+).(\d+)$/;
+            my ($x, $y, $z);
+            if ( /^(\d{2,})/) {
+                ($x, $y, $z ) = ($1, 0, 0);             #  >= v10
+            }
+            else {
+                ($x, $y, $z) = /^(\d)\.(\d+).(\d+)$/;   #  <  v10
+            }
             # Return false if we didn't get all three.
             return unless $x and defined $y and defined $z;
             # Save all three parts.
@@ -350,7 +366,7 @@ sub version {
             # Return true.
             return 1;
         };
-        $self->{version} = $self->unknown( key      => 'version number',
+        $self->{version} = $self->unknown( key     => 'postgres version number',
                                            callback => $chk_version);
     }
 
@@ -365,7 +381,7 @@ sub version {
 
 Returns the PostgreSQL major version number. App::Info::RDBMS::PostgreSQL
 parses the major version number from the system call C<`pg_config --version`>.
-For example, C<version()> returns "7.1.2", then this method returns "7".
+For example, if C<version()> returns "7.1.2", then this method returns "7".
 
 B<Events:>
 
@@ -403,7 +419,7 @@ sub major_version {
     # Load data.
     $get_version->($self) unless exists $self->{'--version'};
     # Handle an unknown value.
-    $self->{major} = $self->unknown( key      => 'major version number',
+    $self->{major} = $self->unknown( key      => 'postgres major version number',
                                      callback => $is_int)
       unless $self->{major};
     return $self->{major};
@@ -451,7 +467,7 @@ sub minor_version {
     # Load data.
     $get_version->($self) unless exists $self->{'--version'};
     # Handle an unknown value.
-    $self->{minor} = $self->unknown( key      => 'minor version number',
+    $self->{minor} = $self->unknown( key      => 'postgres minor version number',
                                      callback => $is_int)
       unless defined $self->{minor};
     return $self->{minor};
@@ -499,7 +515,7 @@ sub patch_version {
     # Load data.
     $get_version->($self) unless exists $self->{'--version'};
     # Handle an unknown value.
-    $self->{patch} = $self->unknown( key      => 'patch version number',
+    $self->{patch} = $self->unknown( key      => 'postgres patch version number',
                                      callback => $is_int)
       unless defined $self->{patch};
     return $self->{patch};
@@ -554,7 +570,7 @@ my $find_exe = sub  {
         if (my $exe = $u->first_cat_exe([$self->$meth(), $exe], $bin)) {
             # We found it. Confirm.
             $self->{$key} = $self->confirm(
-                key      => $key,
+                key      => "path to $key",
                 prompt   => "Path to $key executable?",
                 value    => $exe,
                 callback => sub { -x },
@@ -563,7 +579,7 @@ my $find_exe = sub  {
         } else {
             # Handle an unknown value.
             $self->{$key} = $self->unknown(
-                key      => $key,
+                key      => "path to $key",
                 prompt   => "Path to $key executable?",
                 callback => sub { -x },
                 error    => 'Not an executable'
@@ -624,7 +640,7 @@ sub bin_dir {
         } else {
             # Handle an unknown value.
             $self->error("Cannot find bin directory");
-            $self->{bin_dir} = $self->unknown( key      => 'bin directory',
+            $self->{bin_dir} = $self->unknown( key      => 'postgres bin dir',
                                                callback => $is_dir)
         }
     }
@@ -670,7 +686,7 @@ sub inc_dir {
         } else {
             # Handle an unknown value.
             $self->error("Cannot find include directory");
-            $self->{inc_dir} = $self->unknown( key      => 'include directory',
+            $self->{inc_dir} = $self->unknown( key      => 'postgres include dir',
                                                callback => $is_dir)
         }
     }
@@ -716,7 +732,7 @@ sub lib_dir {
         } else {
             # Handle an unknown value.
             $self->error("Cannot find library directory");
-            $self->{lib_dir} = $self->unknown( key      => 'library directory',
+            $self->{lib_dir} = $self->unknown( key      => 'postgres library dir',
                                                callback => $is_dir)
         }
     }
@@ -765,7 +781,7 @@ sub so_lib_dir {
             # Handle an unknown value.
             $self->error("Cannot find shared object library directory");
             $self->{so_lib_dir} =
-              $self->unknown( key      => 'shared object library directory',
+              $self->unknown( key      => 'postgres so directory',
                               callback => $is_dir)
         }
     }
@@ -1031,14 +1047,19 @@ Path to executable?
 1;
 __END__
 
-=head1 BUGS
+=head1 SUPPORT
 
-Please send bug reports to <bug-app-info@rt.cpan.org> or file them at
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=App-Info>.
+This module is stored in an open L<GitHub
+repository|http://github.com/theory/app-info/>. Feel free to fork and
+contribute!
+
+Please file bug reports via L<GitHub
+Issues|http://github.com/theory/app-info/issues/> or by sending mail to
+L<bug-App-Info@rt.cpan.org|mailto:bug-App-Info@rt.cpan.org>.
 
 =head1 AUTHOR
 
-David Wheeler <david@justatheory.com> based on code by Sam Tregar
+David E. Wheeler <david@justatheory.com> based on code by Sam Tregar
 <sam@tregar.com>.
 
 =head1 SEE ALSO
@@ -1055,7 +1076,7 @@ L<http://www.postgresql.org/> is the PostgreSQL home page.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2002-2004, David Wheeler. All Rights Reserved.
+Copyright (c) 2002-2011, David E. Wheeler. Some Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
